package ca.mcgill.sable.abc.impact.visualizer.actions;

import org.eclipse.core.resources.IFile;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.texteditor.ITextEditor;

import polyglot.util.Position;
import ca.mcgill.sable.abc.impact.visualizer.model.IActCollapse;
import ca.mcgill.sable.abc.impact.visualizer.model.IActJavaElement;
import ca.mcgill.sable.abc.impact.visualizer.model.IConvertibleIntoJavaElement;
import ca.mcgill.sable.abc.impact.visualizer.model.ITreeNode;
import ca.mcgill.sable.abc.impact.visualizer.model.IActPosition;
import ca.mcgill.sable.abc.impact.visualizer.util.ImpactPluginUtil;

/**
 * Acting by test the node following the order:
 * 1. if an {@link IActCollapse} node
 * 2. if an {@link IActPosition} node
 * 3. if an {@link IActJavaElement} node
 * @author Dehua Zhang
 *
 */
public class DoubleClickActionDelegate {
	
	public static void actOn(DoubleClickEvent event) {
		Object element = ((IStructuredSelection)event.getSelection()).getFirstElement();
		if (! (element instanceof ITreeNode)) return;
		
		ITreeNode node = (ITreeNode)element;
		
		if (node instanceof IActCollapse) {
			Viewer viewer = event.getViewer();
			if (viewer instanceof TreeViewer) { //defensive check
				TreeViewer treeViewer = (TreeViewer)event.getViewer();
				if (treeViewer.getExpandedState(node)) {
					treeViewer.collapseToLevel(node, 1);
				} else {
					treeViewer.expandToLevel(node, 1);
				}
			}
		} else if (node instanceof IActPosition) {
			IActPosition ra = (IActPosition)node;
			if (ra != null) {
				openEditorAndReveal(ra.getRevealData().getFile(), ra.getRevealData().getPosition());
			}
		} else if (node instanceof IActJavaElement) {
			IConvertibleIntoJavaElement cj = (IConvertibleIntoJavaElement)node;
			IJavaElement jElement = cj.getJavaElement();
			if (jElement != null) {
				openEditorAndReveal(jElement);
			}
		}
	}
	
	private static IEditorPart openEditor(IFile file) {
		IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
		IEditorPart editorPart = null;
		try {
			editorPart = IDE.openEditor(page, file, true);
		} catch (PartInitException e) {
			e.printStackTrace();
		}
		return editorPart;
	}

	private static void openEditorAndReveal(IFile file, Position pos) {
		IEditorPart editorPart = openEditor(file);
		if (editorPart != null) {
			ITextEditor editor = (ITextEditor) editorPart.getAdapter(ITextEditor.class);
			IRegion region = ImpactPluginUtil.positionToRegion(editor, pos);
			int start = region.getOffset();
			int length = region.getLength();
			editor.selectAndReveal(start, length);
		}
	}
	
	private static void openEditorAndReveal(IJavaElement element) {
		try {
			IEditorPart editorPart = JavaUI.openInEditor(element);
			JavaUI.revealInEditor(editorPart, element);
		} catch (PartInitException e) {
			e.printStackTrace();
		} catch (JavaModelException e) {
			e.printStackTrace();
		}
	}
}
