package ca.mcgill.sable.abc.impact.visualizer.model;

import java.util.LinkedList;
import java.util.List;

public abstract class AbstractTreeVisitor {
	private ITreeNode root;
	
	abstract protected void action(ITreeNode node);
	
	public AbstractTreeVisitor(ITreeNode root) {
		this.root = root;
	}
	
	public void depthPreOrderTraverse() {
		depthPreOrderTraverse(root);
	}
	
	private void depthPreOrderTraverse(ITreeNode node) {
		action(node);
		if (node.hasChildren()) {
			List<ITreeNode> children = node.getChildren();
			for (ITreeNode child : children) {
				depthPreOrderTraverse(child);
			}
		}
	}
	
	public void depthPostOrderTraverse() {
		depthPostOrderTraverse(root);
	}
	
	private void depthPostOrderTraverse(ITreeNode node) {
		if (node.hasChildren()) {
			List<ITreeNode> children = node.getChildren();
			for (ITreeNode child : children) {
				depthPostOrderTraverse(child);
			}
		}
		action(node);
	}
	
	public void breadthTraverse() {
		List<ITreeNode> list = new LinkedList<ITreeNode>();
		list.add(root);
		while (! list.isEmpty()) {
			ITreeNode head = list.get(0);
			list.remove(0);
			list.addAll(head.getChildren());
			action(head);
		}
	}
}
