package ca.mcgill.sable.abc.impact.visualizer.model;

import java.lang.reflect.Modifier;

import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.core.IField;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.ui.JavaUI;

import soot.SootClass;
import soot.SootField;
import abc.impact.impact.ShadowingImpact;
import abc.impact.utils.ImpactUtil;
import ca.mcgill.sable.abc.impact.visualizer.actions.RunImpactAnalysisActionDelegate;
import ca.mcgill.sable.abc.impact.visualizer.util.ImpactPluginUtil;

//NOTE this class is used in plug-in decrators extension
public final class ShadowingImpactOriginTypeTreeNode extends AbstractTreeNode 
	implements IActJavaElement, IProvideJavaElementImage {

	final private ShadowingImpact shadowingImpact;
	final private SootClass originType;
	private final IJavaElement jEle;
	
	public ShadowingImpactOriginTypeTreeNode(ITreeNode parent, final ShadowingImpact shadowingImpact, final SootClass originType) {
		super(parent);
		this.shadowingImpact = shadowingImpact;
		this.originType = originType;
		this.name = originType.getName() + "." + shadowingImpact.getFieldName();

		int mod = Modifier.PUBLIC;
		try {
			SootField sf = originType.getField(shadowingImpact.getFieldName());
			mod = ImpactUtil.getRealModifiers(sf);
		} catch (Exception e) {}
		if (Modifier.isPublic(mod)) {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_FIELD_PUBLIC);
		} else if (Modifier.isProtected(mod)) {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_FIELD_PROTECTED);
		} else if (Modifier.isPrivate(mod)) {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_FIELD_PRIVATE);
		} else {
			this.image = JavaUI.getSharedImages().getImage(
					org.eclipse.jdt.ui.ISharedImages.IMG_FIELD_DEFAULT);
		}
		
		IJavaElement ret = null;
		IType jType = (IType)ImpactPluginUtil.findJavaType(
				RunImpactAnalysisActionDelegate.getCurrentProject(), 
				originType.getName());
		if (jType != null) {
			IField jField = jType.getField(shadowingImpact.getFieldName());
			if (jField.exists()) ret = jField;
			else ret = jType;
		}
		this.jEle = ret;
	}

	public ShadowingImpact getShadowingImpact() {
		return shadowingImpact;
	}

	public SootClass getOriginType() {
		return originType;
	}

	public IJavaElement getJavaElement() {
		return jEle;
	}

	public Object getAdapter(Class adapter) {
		return Platform.getAdapterManager().getAdapter(this, adapter);
	}

}
